/* =====================================================================
 *  MOVIE.c - Animation manager
 * =====================================================================
 * $Revision: 1.13 $
 * $Date: 1994/01/04 19:33:23 $
 * $Author: hidori $
 */

static const char rcsid [] = "$Id: MOVIE.c,v 1.13 1994/01/04 19:33:23 hidori Exp hidori $";

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <sxlib.h>

#include "audio.h"

#include "MOVIE.h"

/* =====================================================================
 *  const
 * =====================================================================
 */

#define SHIRINKFACTOR (1.3)			/* k		*/

/* =====================================================================
 *  static functions
 * =====================================================================
 */

/* =====================================================================
 *  __movieFillScreen  - [rwi`
 * =====================================================================
 *  return value : Ȃ
 */

static void
__movieFillScreen (rect* rct)
{
  int pages = GMAPage (0x0f);
  int color = GMForeColor (G_LGRAY);

  GMFillRect (rct);
  GMForeColor (color);
  GMAPage (pages);
}

/* =====================================================================
 *  pmax - a,b ̑傫
 * =====================================================================
 */

static int
pmax (short* a, short* b)
{
  return *a - *b;
}

/* =====================================================================
 *  combsort - ǃou\[g (Shell sort ̕ό`Ƃv)
 * =====================================================================
 */

static void
combsort (void* base, size_t num, size_t width, int (*compare)(void* elem1, void* elem2))
{
  size_t gap = num;
  int swapped = 0;

  do {
    gap = (int) max ((float) gap / SHIRINKFACTOR, 1);
    switch (gap) {
    case 9:
    case 10:
      gap = 11;
      break;
    }

    {
      int i, top = num - gap;

      swapped = 0;
      for (i = 0; i < top; i++) {
	char* p = (char*) base + (i * width);
	char* q = (char*) base + ((i + gap) * width);

	if (compare (p,q) > 0) {
	  char tmp [width];

	  memcpy (tmp, p,   width);
	  memcpy (p,   q,   width);
	  memcpy (q,   tmp, width);
	  swapped++;
	}
      }
    }
  } while (swapped || (gap > 1));
}

/* =====================================================================
 *  functions
 * =====================================================================
 */

/* =====================================================================
 *  movieOpen - [r̃I[v
 * =====================================================================
 *  return value : NULLȊO ... [r̃nh
 *                 NULL ....... [r̃I[vɎs
 */

MOVIE**
movieOpen (char* fn, graph* gPtr, rect* rct, const int md)
{
  MOVIE** movieHdl = (MOVIE**) MMChHdlNew (sizeof (MOVIE));
  MOVIE* moviePtr;

  /* wĩNA			*/

  {
    graph* lastGraph = GMGetGraph ();

    GMSetGraph (gPtr);
    __movieFillScreen (rct);
    GMSetGraph (lastGraph);
  }

  /* "[r" nh̏		*/

  if (!movieHdl) return NULL;

  handleLock ((handle) movieHdl);
  moviePtr = *movieHdl;

  memset (moviePtr, 0, sizeof (MOVIE));

  /* "[r" t@C̃I[v	*/

  {
    moviePtr->resHdl = TSResOpen (fn);
    if (!moviePtr->resHdl) {
      TSErrDialogN (1, "Can't open \"MOVIE\".");
      handleDispose ((handle) movieHdl);
      return NULL;
    }
  }

  /* 'mTCH' ^C`[g擾		*/

  {
    mTimeChart** mTchHdl;

    mTchHdl = (mTimeChart**) TSRscGet ('mTCH', 128);
    if (!mTchHdl) {
      int cells;
      short** ids;

      moviePtr->pages = 4;
      if (TSResIDList (&cells, &ids, moviePtr->resHdl, 'PAT4') < 0) {
	if (TSResIDList (&cells, &ids, moviePtr->resHdl, 'PAT3') < 0) {
	  if (TSResIDList (&cells, &ids, moviePtr->resHdl, 'PAT2') < 0) {
	    if (TSResIDList (&cells, &ids, moviePtr->resHdl, 'PAT1') < 0) {
	      TSErrDialogN (1, "Can't make scedule.");
	      resourceDispose (moviePtr->resHdl);
	      handleDispose ((handle) movieHdl);
	      return NULL;
	    } else {
	      moviePtr->pages = 1;
	    }
	  } else {
	    moviePtr->pages = 2;
	  }
	} else {
	  moviePtr->pages = 3;
	}
      }
      handleLock (ids);
      combsort (*ids, cells, sizeof (short), &pmax);
      handleUnlock (ids);

      moviePtr->mTch  = ids;
      moviePtr->cells = cells;
    } else {
      moviePtr->mTch = mTchHdl;
      TSRscDetach ((handle) moviePtr->mTch);
    }
  }

  /* ttlImg 擾			*/

  {
    rectImg** ttlImgHdl;

    ttlImgHdl = (rectImg**) TSRscGet ('PAT4', 128);
    if (!ttlImgHdl) {
      ttlImgHdl = (rectImg**) TSRscGet ('PAT3', 128);
      if (!ttlImgHdl) {
	ttlImgHdl = (rectImg**) TSRscGet ('PAT2', 128);
	if (!ttlImgHdl) {
	  ttlImgHdl = (rectImg**) TSRscGet ('PAT1', 128);
	  if (!ttlImgHdl) {
	    TSErrDialogN (1, "Can't get screen size.");
	    resourceDispose (moviePtr->resHdl);
	    handleDispose ((handle) moviePtr->mTch);
	    handleDispose ((handle) movieHdl);
	    return NULL;
	  }
	}
      }
    }

    moviePtr->bounds  = *(rect*) (*ttlImgHdl);
    moviePtr->ttlImgID = 128;

    moviePtr->ttlImgHdl = (handle) ttlImgHdl;
    TSRscDetach ((handle) moviePtr->ttlImgHdl);
  }

  /* lݒ				*/

  moviePtr->mGraph = gPtr;
  moviePtr->mRect  = *rct;

  moviePtr->repeat = 1;

  moviePtr->curImgID  = -1;
  moviePtr->curImgHdl = NULL;

  moviePtr->curSndID  = -1;
  moviePtr->curSndHdl = NULL;

  handleUnlock ((handle) movieHdl);

  return movieHdl;
}

/* =====================================================================
 *  movieDispose - [r̔j
 * =====================================================================
 *  return value : Ȃ
 */

void
movieDispose (MOVIE** movieHdl)
{
  MOVIE* moviePtr;

  if (movieHdl) {
    handleLock ((handle) movieHdl);
    moviePtr = *movieHdl;

    resourceDispose (moviePtr->resHdl);

    handleDispose ((handle) moviePtr->mTch);
    handleDispose ((handle) moviePtr->ttlImgHdl);
    handleDispose ((handle) moviePtr->curImgHdl);
    audioDispose ((handle) moviePtr->curSndHdl);

    handleDispose ((handle) movieHdl);
  }

  return;
}

/* =====================================================================
 *  movieDraw - [r̕`
 * =====================================================================
 *  return value : Ȃ
 */

void
movieDraw (MOVIE** movieHdl, tsevent* eventRecord)
{
  void movieDrawByTch (MOVIE** movieHdl, tsevent* eventRecord);
  void movieDrawBySmt (MOVIE** movieHdl, tsevent* eventRecord);

  if (!movieHdl) return;

  switch ((*movieHdl)->pages) {
  case 0:
    movieDrawByTch (movieHdl, eventRecord);
    break;
  case 2:
  case 3:
  case 4:
    movieDrawBySmt (movieHdl, eventRecord);
    break;
  }
}

void
movieDrawByTch (MOVIE** movieHdl, tsevent* eventRecord)
{
  MOVIE* moviePtr;
  mTimeChart* mTchPtr;

  int magic = 0;

  if (!movieHdl) return;

  /* nȟŒ			*/

  handleLock ((handle) movieHdl);
  moviePtr = *movieHdl;

  handleLock ((handle) moviePtr->mTch);
  mTchPtr = *(moviePtr->mTch);

  GMSetGraph (moviePtr->mGraph);

  if (moviePtr->play) {
    /* O̕`悩̌oߎԂ̎Zo	*/

    if (!moviePtr->pause) {
      moviePtr->runs += (EMSysTime () - moviePtr->prev);
    }
    moviePtr->prev = EMSysTime ();

    if (moviePtr->runs >= mTchPtr->times) {	/* PH				*/
      moviePtr->runs = 0;
      if (!moviePtr->repeat) {			/* s[g[hH			*/
	moviePtr->play  = 0;
	moviePtr->pause = 0;
	magic = 1;
      }
    }

    /* ĐXPW[̎s		*/

    if (!magic && (moviePtr->play && !moviePtr->hide)) {
      int i;

      for (i = 0; i < mTchPtr->tracks; i++) {
	int chop = 10;
	size_t chunkbits = mTchPtr->times / chop;
	size_t chunksize = sizeof (long) + sizeof (mChunk_SKEL) * chunkbits;
	size_t chunkofst = chunksize * i;
	mTrack* dTrack = (mTrack*) (mTchPtr->chunk + chunkofst);

	switch (dTrack->attrib) {
	case 'p':				/* 'PAT4' ̕`			*/
	case 'p4':				/* 'PAT4' ̕`			*/
	case 'p3':				/* 'PAT3' ̕`			*/
	case 'p2':				/* 'PAT2' ̕`			*/
	  {
	    unsigned long type = 'PAT4';
	    short id = dTrack->chunk [moviePtr->runs / chop].pat4.id;
	    unsigned short pages = 7;

	    switch (dTrack->attrib) {
	    case 'p3':
	      pages = 3;
	      type = 'PAT3';
	      break;
	    case 'p2':
	      pages = 3;
	      type = 'PAT2';
	      break;
	    }

	    if (moviePtr->curImgID != id) {	/* rectImg ͊؂ꂩH		*/
	      handleDispose (moviePtr->curImgHdl);

	      TSCurResSet (moviePtr->resHdl);
	      moviePtr->curImgHdl = TSRscGet (type, id);
	      if (moviePtr->curImgHdl) {
		TSRscDetach (moviePtr->curImgHdl);
	      }
	    }

	    if (moviePtr->curImgID != id || eventRecord->what == E_UPDATE) {
	      int lastPages = GMAPage (pages);

	      GMPutRImg ((rectImg*) *(moviePtr->curImgHdl), 0);
	      GMAPage (lastPages);
	    }

	    moviePtr->curImgID = id;
	  }
	  break;
	case 's':				/* 'BEEP' Đ				*/
	  {
	    unsigned long type = 'BEEP';
	    short id = dTrack->chunk [moviePtr->runs / chop].beep.id;

	    if (id) {
	      TSCurResSet (moviePtr->resHdl);

	      {
		handle pcmHdl = TSRscGet (type, id);

		if (pcmHdl) {
		  TSRscDetach (pcmHdl);
		  audioDispose (moviePtr->curSndHdl);
		  moviePtr->curSndHdl = audioCreate (pcmHdl);
		  audioPlay (moviePtr->curSndHdl);
		}
	      }
	    }

	    moviePtr->curSndID = id;
	  }
	  break;
	}
      }
    }
  }

  if (magic || ((eventRecord->what == E_UPDATE) && !moviePtr->play)) {
    if (moviePtr->ttlImgHdl) {
      int pages = GMAPage (7);

      GMPutRImg ((rectImg*) *(moviePtr->ttlImgHdl), 0);
      GMAPage (pages);
    } else {
      __movieFillScreen (&(moviePtr->mRect));
    }
  }

  handleUnlock ((handle) moviePtr->mTch);
  handleUnlock ((handle) movieHdl);

  return;
}

void
movieDrawBySmt (MOVIE** movieHdl, tsevent* eventRecord)
{
  MOVIE* moviePtr;
  mTimeChart* mTchPtr;

  int magic = 0;

  if (!movieHdl) return;

  /* nȟŒ			*/

  handleLock ((handle) movieHdl);
  moviePtr = *movieHdl;

  handleLock ((handle) moviePtr->mTch);
  mTchPtr = *(moviePtr->mTch);

  GMSetGraph (moviePtr->mGraph);

  if (moviePtr->play) {
    /* O̕`悩̌oߎԂ̎Zo	*/

    if (!moviePtr->pause) {
      moviePtr->runs += (EMSysTime () - moviePtr->prev);
    }
    moviePtr->prev = EMSysTime ();

    if (moviePtr->runs >= (moviePtr->cells * 10)) {
      moviePtr->runs = 0;
      if (!moviePtr->repeat) {			/* s[g[hH			*/
	moviePtr->play  = 0;
	moviePtr->pause = 0;
	magic = 1;
      }
    }

    /* ĐXPW[̎s		*/

    if (!magic && (moviePtr->play && !moviePtr->hide)) {
      unsigned long type = 'PAT4';
      short id = ((short*) *(moviePtr->mTch)) [moviePtr->runs / 10];
      unsigned short pages = 7;

      switch (moviePtr->pages) {
      case 3:
	type  = 'PAT3';
	pages = 3;
	break;
      case 2:
	type  = 'PAT2';
	pages = 3;
	break;
      case 1:
	type  = 'PAT1';
	pages = 1;
	break;
      }

      if (moviePtr->curImgID != id) {	/* rectImg ͊؂ꂩH		*/
	handleDispose (moviePtr->curImgHdl);

	TSCurResSet (moviePtr->resHdl);
	moviePtr->curImgHdl = TSRscGet (type, id);
	if (moviePtr->curImgHdl) {
	  TSRscDetach (moviePtr->curImgHdl);
	}
      }

      if (moviePtr->curImgID != id || eventRecord->what == E_UPDATE) {
	int lastPages = GMAPage (pages);

	GMPutRImg ((rectImg*) *(moviePtr->curImgHdl), 0);
	GMAPage (lastPages);
      }

      moviePtr->curImgID = id;
    }
     

    if (magic || ((eventRecord->what == E_UPDATE) && !moviePtr->play)) {
      if (moviePtr->ttlImgHdl) {
	int pages = GMAPage (7);

	GMPutRImg ((rectImg*) *(moviePtr->ttlImgHdl), 0);
	GMAPage (pages);
      } else {
	__movieFillScreen (&(moviePtr->mRect));
      }
    }
  }

  handleUnlock ((handle) moviePtr->mTch);
  handleUnlock ((handle) movieHdl);

  return;
}

/* =====================================================================
 *  moviePlay - [r̍ĐJn
 * =====================================================================
 *  return value : Ȃ
 */

void
moviePlay (MOVIE** movieHdl)
{
  MOVIE* moviePtr;

  if (movieHdl) {
    MMHdlLock ((handle) movieHdl);

    moviePtr = *movieHdl;

    {
      graph* lastGraph = GMGetGraph ();

      GMSetGraph (moviePtr->mGraph);
      __movieFillScreen (&(moviePtr->mRect));
      GMSetGraph (lastGraph);
    }

    moviePtr->play  = 1;
    moviePtr->pause = 0;

    moviePtr->runs = 0;
    moviePtr->prev = EMSysTime ();

    MMHdlUnlock ((handle) movieHdl);
  }

  return;
}

/* =====================================================================
 *  movieStop - [r̍Đ~
 * =====================================================================
 *  return value : Ȃ
 */

void
movieStop (MOVIE** movieHdl)
{
  if (movieHdl) {
    (*movieHdl)->play  = 0;
    (*movieHdl)->pause = 0;
  }

  return;
}

/* =====================================================================
 *  moviePlayInvert - [r̍ĐԔ]
 * =====================================================================
 *  return value : Ȃ
 */

void
moviePlayInvert (MOVIE** movieHdl)
{
  if (movieHdl) {
    (*movieHdl)->play  ? movieStop (movieHdl) : moviePlay (movieHdl);
  }

  return;
}

/* =====================================================================
 *  moviePause - [r̃|[Yݒ
 * =====================================================================
 *  return value : Ȃ
 */

void
moviePause (MOVIE** movieHdl)
{
  if (movieHdl && (*movieHdl)->play) {
    (*movieHdl)->pause = 1;
  }

  return;
}

/* =====================================================================
 *  movieUnPause - [r̃|[Y
 * =====================================================================
 *  return value : Ȃ
 */

void
movieUnPause (MOVIE** movieHdl)
{
  if (movieHdl && (*movieHdl)->play) {
    (*movieHdl)->pause = 0;
  }

  return;
}

/* =====================================================================
 *  moviePauseInvert - [r̃|[YԔ]
 * =====================================================================
 *  return value : Ȃ
 */

void
moviePauseInvert (MOVIE** movieHdl)
{
  if (movieHdl && (*movieHdl)->play) {
    (*movieHdl)->pause = !((*movieHdl)->pause);
  }

  return;
}

/* =====================================================================
 *  movieRepeat - [r̃s[gݒ
 * =====================================================================
 *  return value : Ȃ
 */

void
movieRepeat (MOVIE** movieHdl)
{
  if (movieHdl) {
    (*movieHdl)->repeat = 1;
  }

  return;
}

/* =====================================================================
 *  movieUnRepeat - [r̃s[g
 * =====================================================================
 *  return value : Ȃ
 */

void
movieUnRepeat (MOVIE** movieHdl)
{
  if (movieHdl) {
    (*movieHdl)->repeat = 0;
  }

  return;
}

/* =====================================================================
 *  movieRepeatInvert - [r̃s[gԔ]
 * =====================================================================
 *  return value : Ȃ
 */

void
movieRepeatInvert (MOVIE** movieHdl)
{
  if (movieHdl) {
    (*movieHdl)->repeat = !((*movieHdl)->repeat);
  }

  return;
}

/* =====================================================================
 *  movieShow - [r̉
 * =====================================================================
 *  return value : Ȃ
 */

void
movieShow (MOVIE** movieHdl)
{
  if (movieHdl) {
    (*movieHdl)->hide = 0;
  }

  return;
}

/* =====================================================================
 *  movieHide - [r̕s
 * =====================================================================
 *  return value : Ȃ
 */

void
movieHide (MOVIE** movieHdl)
{
  if (movieHdl) {
    (*movieHdl)->hide = 1;
  }

  return;
}

/* =====================================================================
 *  movieGetStatusStr - [rĐԂ\𓾂
 * =====================================================================
 */

char*
movieGetStatusStr (MOVIE** movieHdl)
{
  static char statStr [32];

  *statStr = 0;
  if (movieHdl) {
    sprintf (
	     statStr,
	     "%d%d%d%d",
	     !!(*movieHdl)->play,
	     !!(*movieHdl)->pause,
	     !!(*movieHdl)->hide,
	     !!(*movieHdl)->repeat
	     );
  }

  return statStr;
}

/* =====================================================================
 *  movieSetStatusStr - [rĐԂ\Ŏw肷
 * =====================================================================
 */

void
movieSetStatusStr (MOVIE** movieHdl, const char* statStr)
{
  if (movieHdl) {
    switch (strlen (statStr)) {
    case 4:
      (*movieHdl)->repeat = (statStr [3] > '0');
    case 3:
      (*movieHdl)->hide   = (statStr [2] > '0');
    case 2:
      (*movieHdl)->pause  = (statStr [1] > '0');
    case 1:
      (*movieHdl)->play   = (statStr [0] > '0');
      break;
    }
  }
}

/* =====================================================================
 *  end of MOVIE.c
 * =====================================================================
 * $Log: MOVIE.c,v $
 * Revision 1.13  1994/01/04  19:33:23  hidori
 * EI[fBIĐɎbΉ
 *
 * Revision 1.12  1994/01/03  08:00:32  hidori
 * E'mTCH' Ȃ .FLM ̍ăT|[g
 *
 * Revision 1.11  1994/01/01  11:06:36  hidori
 * *** empty log message ***
 *
 * Revision 1.10  1993/08/22  14:42:09  hidori
 * E|[ỸAbvf[g̉P
 *
 * Revision 1.9  1993/08/17  15:58:47  hidori
 * E|[YԂ̃Abvf[gꕔύX
 *
 * Revision 1.8  1993/08/16  16:35:40  hidori
 * E] lock, unlock ̍폜
 * EnhQƂ̈ꕔ|C^QƂɕύXA}
 *
 * Revision 1.7  1993/08/16  13:41:59  hidori
 * EĐ~,ĐԔ]̒ǉ
 *
 * Revision 1.6  1993/08/14  17:55:32  hidori
 * EĐԎ擾^ݒ胁\bh̒ǉ
 *
 * Revision 1.5  1993/08/14  13:01:14  hidori
 * EnCh̎
 *
 * Revision 1.4  1993/08/14  12:15:28  hidori
 * E|[YAs[gԎw̎
 *
 * Revision 1.3  1993/08/12  16:34:06  hidori
 * E'mTCH' ͕̋
 *
 * Revision 1.2  1993/08/11  11:16:06  hidori
 * E^C`[g̎QƂ
 * E|[Y̎
 *
 * Revision 1.1  1993/08/05  13:51:09  hidori
 * Initial revision
 *
 */
