/* =====================================================================
 *  audio.c - Extended audio handling methods
 * =====================================================================
 * $Reivision$
 * $Date: 1994/01/04 19:33:23 $
 * $Author: hidori $
 */

static const char rcsid [] = "$Id: audio.c,v 1.1 1994/01/04 19:33:23 hidori Exp hidori $";

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/dos.h>
#include <sys/iocs.h>

#include <sxlib.h>

#include "audio.h"

/* =====================================================================
 *  macros
 * =====================================================================
 */

#define handleNew(size)		(MMChHdlNew ((size)))
#define handleDispose(hdl)	((hdl) ? MMHdlDispose ((hdl)) : 0)
#define handleLock(hdl)		((hdl) ? MMHdlLock ((hdl))    : 0)
#define handleUnlock(hdl)	((hdl) ? MMHdlUnlock ((hdl))  : 0)
#define handleSizeGet(hdl)	((hdl) ? MMHdlSizeGet ((hdl)) : -1)

/* =====================================================================
 *  functions
 * =====================================================================
 */

/* =====================================================================
 *  audioCreate - I[fBIR[h̍쐬
 * =====================================================================
 */

audio_t**
audioCreate (handle bufHdl)
{
  if (bufHdl) {
    long size = handleSizeGet (bufHdl);
    long chunks = (size / 780) + !!(size % 780);
    struct _chain2** tblHdl = handleNew (chunks * sizeof (struct _chain2));
    audio_t** pcmHdl = handleNew (sizeof (audio_t));

    if (tblHdl && pcmHdl && size) {
      handleLock (bufHdl);
      handleLock (tblHdl);
      handleLock (pcmHdl);

      memset (*pcmHdl, 0, sizeof (audio_t));

      (*pcmHdl)->bufHdl = bufHdl;
      (*pcmHdl)->tblHdl = tblHdl;
      (*pcmHdl)->chunks = chunks;
      (*pcmHdl)->level  = 20;

      {
	int i;

	for (i = 0; i < chunks; i++) {
	  (*tblHdl) [i].addr = ((char*) (*bufHdl)) + (i * 780);
	  (*tblHdl) [i].len  = 780;
	  (*tblHdl) [i].next = *tblHdl + i + 1;
	}

	(*tblHdl) [i - 1].len  = ((size % 780) ? (size % 780) : 780);
	(*tblHdl) [i - 1].next = NULL;
      }

      handleUnlock (pcmHdl);

      return pcmHdl;
    }

    handleDispose (tblHdl);
    handleDispose (pcmHdl);
  }

  return NULL;
}

/* =====================================================================
 *  audioPlay - I[fBIĐJn
 * =====================================================================
 */

int
audioPlay (audio_t** audioHdl)
{
  if (!audioHdl || !*audioHdl) return 0;

  if ((*audioHdl)->play) audioStop (audioHdl);

  {
    int sns = _iocs_adpcmsns ();

    if (sns) {
      _iocs_adpcmmod (0);
    }
  }

  {
    struct _chain2** tblHdl = (*audioHdl)->tblHdl;
    long chunks = (*audioHdl)->chunks;

    int i;

    for (i = 0; i < chunks; i++) {
      (*tblHdl) [i].next = *tblHdl + i + 1;
    }

    (*tblHdl) [i - 1].next = NULL;

    _iocs_adpcmlot (*tblHdl, 256 * 4 + 3);

    (*audioHdl)->play = 1;
  }

  return 1;
}

/* =====================================================================
 *  audioStop - I[fBIĐ~
 * =====================================================================
 */

int
audioStop (audio_t** audioHdl)
{
  if (!audioHdl || !*audioHdl) return 0;

  {
    struct _chain2** tblHdl = (*audioHdl)->tblHdl;
    long chunks = (*audioHdl)->chunks;

    int i;

    for (i = 0; i < chunks; i++) {
      (*tblHdl) [i].next = NULL;
    }

    if ((*audioHdl)->play) {
      int sns = _iocs_adpcmsns ();

      if (sns) {
	_iocs_adpcmmod (0);
      }
    }

    (*audioHdl)->play = 0;
  }

  return 1;
}

/* =====================================================================
 *  audioDispose - I[fBIR[h̔p
 * =====================================================================
 */

void
audioDispose (audio_t** audioHdl)
{
  if (!audioHdl || !*audioHdl) return;

  audioStop (audioHdl);

  handleDispose ((*audioHdl)->bufHdl);
  handleDispose ((*audioHdl)->tblHdl);

  handleDispose (audioHdl);
}

#ifdef TEST_AUDIO

/* =====================================================================
 *  static functions
 * =====================================================================
 */

/* =====================================================================
 *  fileToHdl - t@Cnhɓǂݍ
 * =====================================================================
 */

static inline handle
fileToHdl (char* fn)
{
  int fd = TSOpen (fn, 0);

  if (fd >= 0) {
    long size;

    size = _dos_seek (fd, 0, SEEK_END);
    _dos_seek (fd, 0, SEEK_SET);

    {
      handle hdl = handleNew (size);

      if (hdl) {
	long rsize;

	handleLock (hdl);
	rsize = _dos_read (fd, *hdl, size);
	handleUnlock (hdl);
	TSClose (fd);
	if (rsize == size) return hdl;
	handleDispose (hdl);
      }
    }
  }

  return NULL;
}

/* =====================================================================
 *  functions
 * =====================================================================
 */

/* =====================================================================
 *  main - ĂƗpC
 * =====================================================================
 */

const char _sxkernelcomm [] = "sxwin.x";

void
main (int argc, char* argv [])
{
  if (argc < 2) {
    TSErrDialogN (1, "t@Cw肵ĉ");
    exit (0);
  }

  {
    handle pcmHdl = fileToHdl (argv [1]);

    if (!pcmHdl) {
      TSErrDialogN (1, "t@Cǂ߂܂ł");
      exit (0);
    }

    {
      audio_t** audioHdl = audioCreate (pcmHdl);

      if (!audioHdl) {
	TSErrDialogN (1, "I[fBInh쐬ł܂ł");
	exit (0);
      }

      audioPlay (audioHdl);

      {
	long start = EMSysTime ();
	long now = start;
	tsevent event;

	while ((now - start) < (8 * 100)) {
	  TSEventAvail (EM_IDLE, &event);
	  now = event.when;
	}
      }

      audioStop (audioHdl);
      audioDispose (audioHdl);
    }
  }
}

#endif /* TEST_AUDIO */

/* =====================================================================
 *  end of audio.c
 * =====================================================================
 * $Log: audio.c,v $
 * Revision 1.1  1994/01/04  19:33:23  hidori
 * Initial revision
 *
 */
