/* =====================================================================
 *  bit.c - bit field handling utilities
 * =====================================================================
 * $Revision: 1.4 $
 * $Date: 1993/06/26 14:52:43 $
 * $Author: hidori $
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "bit.h"

/* =====================================================================
 *  bitmask - w͈͂ bit ׂ on łl쐬
 * =====================================================================
 */

unsigned long
bitmask (int begin, int end)
{
  unsigned long r = 0;
  int i;

  begin = (begin < 0 ? 0 : begin);
  end   = (end   < 0 ? 0 : end);

  if (end < begin) {
    int tmp = begin;
    begin = end;
    end   = tmp;
  }

  for (i = begin; i <= end; i++) {
    r |= bit (i);
  }

  return r;
}

/* =====================================================================
 *  bitreverse - w͈͂ bit ̕тt]
 * =====================================================================
 */

unsigned long
bitreverse (const unsigned long n, int begin, int end)
{
  unsigned long r = 0;
  int i;

  begin = (begin < 0 ? 0 : begin);
  end   = (end   < 0 ? 0 : end);

  if (end < begin) {
    int tmp = begin;
    begin = end;
    end   = tmp;
  }

  r = (n & ~bitmask (begin, end));

  for (i = begin; i <= (begin + (end - begin) / 2); i++) {
    int p = end - (i - begin);

    r |= (bittest (n, i) ? bit (p) : 0);
    r |= (bittest (n, p) ? bit (i) : 0);
  }

  return r;
}

/* =====================================================================
 *  convertendian - GfBA̕ϊ
 * =====================================================================
 */

unsigned long
convertendian (const unsigned long mdw)
{
  bit_miw miw;
  unsigned short s;
  unsigned char c;

  miw.l = mdw;

  c = miw.c [0];
  miw.c [0] = miw.c [1];
  miw.c [1] = c;

  c = miw.c [2];
  miw.c [2] = miw.c [3];
  miw.c [3] = c;

  s = miw.s [0];
  miw.s [0] = miw.s [1];
  miw.s [1] = s;

  return miw.l;
}

#ifdef BIT_TEST

/* =====================================================================
 *  test - eXgpC
 * =====================================================================
 */

/* =====================================================================
 *  bitpack - bit XJlɕϊ
 * =====================================================================
 */

static unsigned long
bitpack (const char* s)
{
  unsigned long r = 0;

  while (*s) {
    r = (r << 1) | (*s == '1');
    s++;
  }

  return r;
}

/* =====================================================================
 *  bitunpack - XJl bit ɕϊ
 * =====================================================================
 */

static char*
bitunpack (const unsigned long n, char* s)
{
  char* p = s;
  int i;

  for (i = BIT_OF_LWORD - 1; i >= 0; i--) {
    *p++ = (bittest (n, i) ? '1' : '0');
  }
  *p = 0;

  return s;
}

/* =====================================================================
 *  main - eXgpC
 * =====================================================================
 */

void
main (int argc, char* argv [])
{
  unsigned long n;
  char dst [64];

  if (argc < 2) {
    fprintf (stderr, "usage: bit <decimal digit>\n");
    return;
  }

  n = atol (argv [1]);

  printf ("%16s : %s\n", "original",    bitunpack (n, dst));
  printf ("%16s : %s\n", "unpack&pack", bitunpack (bitpack (bitunpack (n, dst)), dst));
  printf ("%16s : %s\n", "mask (2, 6)", bitunpack (bitmask (2, 6), dst));
  printf ("%16s : %s\n", "reverse",     bitunpack (bitreverse (n, 2, 6), dst));
  printf (
	  "%16s : %08X -> %08X\n",
	  "convertendian",
	  0x012345678,
	  convertendian (0x012345678)
	  );

  printf ("\n");
}

#endif /* BIT_TEST */

/* =====================================================================
 *  end of bit.c
 * =====================================================================
 * $Log: bit.c,v $
 * Revision 1.4  1993/06/26  14:52:43  hidori
 * Eꕔ̃fobOp֐ #ifdef ` #endif Ɉړ
 *
 * Revision 1.3  1993/06/21  11:18:43  hidori
 * E֐ύX bitexpand () -> bitunpack ()
 *
 * Revision 1.2  1993/06/09  12:39:56  hidori
 * * bitexpand (), bitreverse () ̕sC
 *
 * Revision 1.1  1993/06/09  12:31:41  hidori
 * Initial revision
 *
 */
