/* =====================================================================
 *  bit.h - bit field handling utilities
 * =====================================================================
 * $Revision: 1.5 $
 * $Date: 1993/06/26 15:39:13 $
 * $Author: hidori $
 */

#ifndef BIT_H
#define BIT_H

/* =====================================================================
 *  const
 * =====================================================================
 */

/* =====================================================================
 *  BIT_BYTE, BIT_WORD, BIT_LWORD - e^̕\ bit 
 * =====================================================================
 */

#define BIT_OF_BYTE	(8)
#define BIT_OF_WORD	(16)
#define BIT_OF_LWORD	(32)

/* =====================================================================
 *  BIT_MASK_BYTE, BIT_MASK_WORD, BIT_MASK_LWORD - e^̃}XNl
 * =====================================================================
 */

#define BIT_MASK_BYTE	(0x0ffUL)
#define BIT_MASK_WORD	(0x0ffffUL)
#define BIT_MASK_LWORD	(0x0ffffffffUL)

/* =====================================================================
 *  macro
 * =====================================================================
 */

/* =====================================================================
 *  bit - w bit  on ł LWORD lԂ
 * =====================================================================
 */

#define bit(n)		(((unsigned long) 1) << (n))

/* =====================================================================
 *  bittest - w bit  on ł邩ǂԂ
 * =====================================================================
 */

#define bittest(n,i)	(!!((unsigned long) (n) & bit(i)))

/* =====================================================================
 *  bit{set,reset} - w bit  {on,off} lԂ
 * =====================================================================
 */

#define bitset(n,i)	((unsgined long) (n) | bit(i))
#define bitreset(n,i)	((unsigned long) (n) & ~bit(i))

/* =====================================================================
 *  byteof, wordof - wl BYTE,WORD o
 * =====================================================================
 */

#define byteof(n)	(((unsigned long) (n)) & BIT_MASK_BYTE)
#define wordof(n)	(((unsigned long) (n)) & BIT_MASK_WORD)

/* =====================================================================
 *  {h,l}byteof - wl {HIGH,LOW} BYTE o
 * =====================================================================
 */

#define hbyteof(n)	(byteof ((unsigned long) (n) >> BIT_OF_BYTE))
#define lbyteof(n)	byteof((n))

/* =====================================================================
 *  {h,l}wordof - wl {HIGH,LOW} WORD o
 * =====================================================================
 */

#define lwordof(n)	wordof((n))
#define hwordof(n)	(wordof ((unsigned long) (n) >> BIT_OF_WORD))

/* =====================================================================
 *  mk{,l}word - h,l  2l {WORD,LWORD} l쐬
 * =====================================================================
 */

#define mkword(h,l)	((byteof(h) << BIT_OF_BYTE) | byteof(l))
#define mklword(h,l)	((wordof(h) << BIT_OF_WORD) | wordof(l))

/* =====================================================================
 *  typedef
 * =====================================================================
 */

/* =====================================================================
 *  bit_miw - }V[hɓƗȃ[h\s߂̒Ԍ`
 * =====================================================================
 */

typedef union bit_miw {
  unsigned long  l;
  unsigned short s [2];
  unsigned char  c [4];
} bit_miw;

/* =====================================================================
 *  functions
 * =====================================================================
 */

unsigned long bitmask (int begin, int end);
unsigned long bitreverse (const unsigned long n, int begin, int end);
unsigned long convertendian (const unsigned long mdw);

#endif /* BIT_H */

/* =====================================================================
 *  end of bit.h
 * =====================================================================
 * $Log: bit.h,v $
 * Revision 1.5  1993/06/26  15:39:13  hidori
 * E}N̏CRގ
 *
 * Revision 1.4  1993/06/26  14:52:43  hidori
 * Eꕔ̃fobOp֐ #ifdef ` #endif Ɉړ
 *
 * Revision 1.3  1993/06/21  11:18:43  hidori
 * E֐ύX bitexpand () -> bitunpack ()
 *
 * Revision 1.2  1993/06/09  12:39:56  hidori
 * * bitexpand (), bitreverse () ̕sC
 *
 */
