/* =====================================================================
 *  filename.c - filename utilities
 * =====================================================================
 * $Revision: 1.3 $
 * $Date: 1993/06/26 14:52:43 $
 * $Author: hidori $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "filename.h"

/* =====================================================================
 *  convertslash - 񒆂 BACKSLASH  SLASH ɒu
 * =====================================================================
 *  return value: dst
 */

char*
convertslash (const char* src, char* dst)
{
  char* p = dst;
  char c;

  while (c = *src++, *dst++ = (c == BACKSLASH  ? SLASH : c));
    
  return p;
}

/* =====================================================================
 *  deleteduplicatedslash - 񒆂̏d SLASH 
 * =====================================================================
 *  return value: dst
 */

char*
deleteduplicatedslash (const char* src, char* dst)
{
  char* p = dst;
  char c = 0;

  while (*dst = *src) {
    if (c != SLASH || *src != SLASH) dst++;
    c = *src++;
  }

  return p;
}

/* =====================================================================
 *  pathname - pX𒊏o
 * =====================================================================
 *  return value: dst
 */

char*
pathname (const char* src, char* dst)
{
  char* p = src + strlen (src);

  strcpy (dst, src);
  while (*p != SLASH && p > src) p--;
  *(dst + ((*p != SLASH ? p : p + 1) - src)) = 0;
  
  return dst;
}

/* =====================================================================
 *  basename - x[Xl[𒊏o
 * =====================================================================
 *  return value: dst
 */

char*
basename (const char* src, char* dst)
{
  char* p = src + strlen (src);

  while (*p != SLASH && p > src) p--;
  strcpy (dst, (*p != SLASH ? p : p + 1));
  
  return dst;
}

/* =====================================================================
 *  suffix - gq𒊏o
 * =====================================================================
 *  return value: dst
 */

char*
suffix (const char* src, char* dst)
{
  char* p = src + strlen (src);

  while (*p != SLASH && *p != '.' && p > src) p--;
  strcpy (dst, (*p != SLASH && *p != '.' ? p : p + 1));
  
  return dst;
}

/* =====================================================================
 *  deletesuffix - gq
 * =====================================================================
 *  return value: dst
 */

char*
deletesuffix (const char* src, char* dst)
{
  char* p = src + strlen (src);

  strcpy (dst, src);
  while (*p != SLASH && p > src) {
    if (*p == '.') {
      *(dst + (p - src)) = 0;
      break;
    }
    p--;
  }
  
  return dst;
}

#ifdef FILENAME_TEST

/* =====================================================================
 *  main - eXgpC
 * =====================================================================
 */

void
main (int argc, char* argv [])
{
  char src [CCMAXPATHCOMP], dst [CCMAXPATHCOMP];

  if (argc < 2) {
    fprintf (stderr, "usage: filename <filename>\n");
    return;
  }

  strcpy (src, argv [1]);

  printf ("%24s: %s\n", "original", src);
  printf ("%24s: %s\n", "convertslash", convertslash (src, src));
  printf ("%24s: %s\n", "deleteduplicatedslash", deleteduplicatedslash (src, src));
  printf ("%24s: %s\n", "pathname", pathname (src, dst));
  printf ("%24s: %s\n", "basename", basename (src, dst));
  printf ("%24s: %s\n", "suffix", suffix (src, dst));
  printf ("%24s: %s\n", "deletesuffix", deletesuffix (src, dst));
  printf ("\n");
}

#endif /* FILENAME_TEST */

/* =====================================================================
 *  end of filename.c
 * =====================================================================
 * $Log: filename.c,v $
 * Revision 1.3  1993/06/26  14:52:43  hidori
 * EƐ
 *
 * Revision 1.2  1993/06/01  15:47:24  hidori
 * * pathname (), basename (), suffix (), deletesuffix () ̒ǉ
 *
 * Revision 1.1  1993/05/23  15:19:18  hidori
 * Initial revision
 *
 */
