/* =====================================================================
 *  window.c - Extended window handling methods
 * =====================================================================
 * $Revision: 1.5 $
 * $Date: 1994/01/01 11:06:36 $
 * $Author: hidori $
 */

static const char rcsid [] = "$Id: window.c,v 1.5 1994/01/01 11:06:36 hidori Exp $";

#define WINDOW_UTILS

#include <stdio.h>
#include <string.h>

#include "window.h"

/* =====================================================================
 *  windowOpen - g window I[v
 * =====================================================================
 * return value : 1 ... window ̃I[vɐ
 *		  0 ... window ̃I[vɎs
 */

int
windowOpen (
	    window_t* wsPtr,
	    int wid,
	    char* title,
	    rect* bounds,
	    rect* grow,
	    int tid,
	    int (**proc)(),
	    int option
	    )
{
  ztol (title, &(wsPtr->title));

  wsPtr->windowPtr = WMOpen (
			     (window*) 0,
			     bounds,
			     &(wsPtr->title),
			     0,
			     wid,
			     (window*) -1,
			     !!(option & WINDOW_OPT_CLOSEBOX),
			     tid
			     );

  if (!wsPtr->windowPtr) return 0;

  if (option & WINDOW_OPT_GROWBOX) {
    (wsPtr->windowPtr)->wOption |= 0x00108;
  }

  wsPtr->boundsRect = *bounds;
  wsPtr->growRect   = (grow ? *grow : (rect) {0,0,0,0});
  wsPtr->activeFlag = 0;
  wsPtr->updateFlag = 1;
  wsPtr->option     = option;
  GMSetGraph ((graph*) wsPtr->windowPtr);
  GMAPage (wsPtr->activePage = GMAPage (0));
  GMForeColor (wsPtr->foreColor = GMForeColor (0));
  GMBackColor (wsPtr->backColor = GMBackColor (0));

  memcpy (wsPtr->proc, proc, sizeof (int (*)()) * (SXEVENTS + WMEXTEND));

  if (!(option & WINDOW_OPT_HIDE)) {
    WMShow (wsPtr->windowPtr);
  }

  return 1;
}

/* =====================================================================
 *  windowClose - g window N[Y
 * =====================================================================
 * return value : 1 ... window ̃N[Yɐ
 *		  0 ... window ̃N[YɎs
 */

int
windowClose (window_t* wsPtr)
{
  int r = 1;

  if (wsPtr->windowPtr) {
    if (wsPtr->proc [WM_DISPOSE]) {
      (wsPtr->proc [WM_DISPOSE]) ();
    }

    CMKill (wsPtr->windowPtr);

    r = !WMDispose (wsPtr->windowPtr);
    memset (wsPtr, 0, sizeof (window_t));
  }
  return 1;
}

/* =====================================================================
 *  windowSetTitle - window ̃^Cgݒ肷
 * =====================================================================
 * return value : 1 ... window ^Cg̐ݒɐ
 *		  0 ... window ^Cg̐ݒɎs
 */

int
windowSetTitle (window_t* wsPtr, char* title)
{
  ztol (title, &(wsPtr->title));
  return !WMTitleSet (wsPtr->windowPtr, &(wsPtr->title));
}

/* =====================================================================
 *  windowShow - window ̉
 * =====================================================================
 */

void
windowShow (window_t* wsPtr)
{
  if (wsPtr->option & WINDOW_OPT_HIDE) {
    wsPtr->option &= (~WINDOW_OPT_HIDE);
    WMShow (wsPtr->windowPtr);
  }
  WMSelect (wsPtr->windowPtr);
}

/* =====================================================================
 *  windowHide - window ̕s
 * =====================================================================
 */

void
windowHide (window_t* wsPtr)
{
  if (!(wsPtr->option & WINDOW_OPT_HIDE)) {
    wsPtr->option |= WINDOW_OPT_HIDE;
    WMHide (wsPtr->windowPtr);
  }
}

/* =====================================================================
 *  windowSaveTdb - task data base XV̏
 * =====================================================================
 */

void
windowSaveTdb (window_t* wsPtr, char* commandStr)
{
  wsPtr->proc [WM_SAVE_TDB] (commandStr);
}

/* =====================================================================
 *  windowGetActiveList - ANeBu window ̃Xg\𓾂
 * =====================================================================
 */

char*
windowGetActiveList (window_t* windowList, int nest)
{
  static char activeList [32];

  int i;

  for (i = 0; i < nest && i < 31; i++) {
    if (windowList [i].windowPtr && windowList [i].windowPtr->wVisible) {
      activeList [i] = '1' + !!(windowList [i].activeFlag);
    } else {
      activeList [i] = '0';
    }
  }
  activeList [i] = 0;

  return activeList;
}
      
/* =====================================================================
 *  end of window.c
 * =====================================================================
 * $Log: window.c,v $
 * Revision 1.5  1994/01/01  11:06:36  hidori
 * *** empty log message ***
 *
 * Revision 1.4  1993/08/15  03:05:40  hidori
 * E}N`̐
 *
 * Revision 1.3  1993/08/14  12:15:28  hidori
 * EWM_SAVE_TDB \bh̒ǉ
 * EShow, Hide \bh̋
 *
 * Revision 1.2  1993/08/12  16:34:06  hidori
 * ECxgvV[WgAEBhEp̏ǉ
 *
 * Revision 1.1  1993/08/05  13:51:09  hidori
 * Initial revision
 *
 */
