/* =====================================================================
 *  window.h - Extended window handling methods
 * =====================================================================
 * $Revision: 1.4 $
 * $Date: 1993/08/14 12:15:28 $
 * $Author: hidori $
 */

#ifndef WINDOW_H
#define WINDOW_H

#include <sxlib.h>

#include "misc.h"

/* =====================================================================
 *  const
 * =====================================================================
 */

/* =====================================================================
 *  number of window procedure
 * =====================================================================
 */

#define SXEVENTS	(16)	/* VXeŒ`Ă event ̐	*/
#define WMEXTEND	(4)	/* window  g proc ̐		*/

/* =====================================================================
 *  window procedure entry
 * =====================================================================
 */

#define WM_DISPOSE	(16)	/* window p̏			*/
#define WM_SAVE_TDB	(17)	/* task data base XV̏		*/

/* =====================================================================
 *  window option
 * =====================================================================
 */

#define WINDOW_OPT_GROWBOX	(0x001)
#define WINDOW_OPT_CLOSEBOX	(0x002)
#define WINDOW_OPT_HIDE		(0x010)

/* =====================================================================
 *  typedef
 * =====================================================================
 */

/* =====================================================================
 *  window_t - g window \
 * =====================================================================
 */

#ifndef WINDOW_T
#define WINDOW_T

typedef struct _window_t {
  window* windowPtr;
  LASCII title;
  rect boundsRect;
  rect growRect;
  short activeFlag;
  short updateFlag;
  short option;
  short activePage;
  short foreColor;
  short backColor;
  int (*proc [SXEVENTS + WMEXTEND])();
} window_t;

#endif /* WINDOW_T */

/* =====================================================================
 *  window_pos 
 * =====================================================================
 */

typedef struct _window_pos {
  short flag;
  point_t pos;
} window_pos;

/* =====================================================================
 *  inline functions
 * =====================================================================
 */

/* =====================================================================
 *  windowWidth - window ̕߂
 * =====================================================================
 *  return: window ̕
 */

static inline int
windowWidth (window_t* p)
{
  return p->windowPtr->wGraph.grRect.right - p->windowPtr->wGraph.grRect.left;
}

/* =====================================================================
 *  windowHeight - window ̍߂
 * =====================================================================
 *  return: window ̍
 */

static inline int
windowHeight (window_t* p)
{
  return p->windowPtr->wGraph.grRect.bottom - p->windowPtr->wGraph.grRect.top;
}

/* =====================================================================
 *  sizeofWindow - window ̕A߂
 * =====================================================================
 *  return: window ̕A(point_t ^)
 */

static inline point_t
sizeofWindow (window_t* p)
{
  point pt = (point) {windowWidth (p), windowHeight (p)};

  return *(point_t*) &pt;
}

/* =====================================================================
 *  windowPos - window ̈ʒu߂
 * =====================================================================
 *  return: window ̈ʒu(point_t ^)
 */

static inline point_t
windowPos (window_t* p)
{
  graph* lastGraph = GMGetGraph ();

  point pt = (point) {
    p->windowPtr->wGraph.grRect.left,
    p->windowPtr->wGraph.grRect.top
  };

  point_t pos;

  GMSetGraph ((graph*) p->windowPtr);
  pos = GMLocalToGlobal (*(point_t*) &pt);
  GMSetGraph (lastGraph);

  return pos;
}

/* =====================================================================
 *  prototype
 * =====================================================================
 */

/* =====================================================================
 *  windowOpen - g window I[v
 * =====================================================================
 */

int windowOpen (
		window_t* wsPtr,
		int wid,
		char* title,
		rect* bounds,
		rect* grow,
		int tid,
		int (**proc)(),
		int option
		);

/* =====================================================================
 *  windowClose - g window N[Y
 * =====================================================================
 */

int windowClose (window_t* wsPtr);

/* =====================================================================
 *  windowSetTitle - window title ݒ肷
 * =====================================================================
 */

int windowSetTitle (window_t* wsPtr, char* title);

/* =====================================================================
 *  windowGetTitle - window title ̎擾
 * =====================================================================
 */

int windowGetTitle (window_t* wsPtr, char* title);

/* =====================================================================
 *  windowShow - window ̉
 * =====================================================================
 */

void windowShow (window_t* wsPtr);

/* =====================================================================
 *  windowHide - window ̕s
 * =====================================================================
 */

void windowHide (window_t* wsPtr);

/* =====================================================================
 *  windowSaveTdb - task data base XV̏
 * =====================================================================
 */

void windowSaveTdb (window_t* wsPtr, char* commandStr);

/* =====================================================================
 *  windowGetActiveList - ANeBu window ̃Xg\𓾂
 * =====================================================================
 */

char* windowGetActiveList (window_t* windowList, int nest);

#endif /* WINDOW_H */

/* =====================================================================
 *  end of window.h
 * =====================================================================
 * $Log: window.h,v $
 * Revision 1.4  1993/08/14  12:15:28  hidori
 * EWM_SAVE_TDB \bh̒ǉ
 * EShow, Hide \bh̋
 *
 * Revision 1.3  1993/08/12  16:34:06  hidori
 * ECxgvV[WgAEBhEp̏ǉ
 *
 * Revision 1.2  1993/08/11  11:16:06  hidori
 * EEBhEI[ṽV[PXꕔ
 *
 * Revision 1.1  1993/08/05  13:51:09  hidori
 * Initial revision
 *
 */

